@@ -3,7 +3,6 @@ package com.unciv.logic.automation.unit
 import com.unciv.Constants
 import com.unciv.UncivGame
 import com.unciv.logic.automation.Automation
-import com.unciv.logic.automation.civilization.NextTurnAutomation
 import com.unciv.logic.battle.Battle
 import com.unciv.logic.battle.BattleDamage
 import com.unciv.logic.battle.CityCombatant
@@ -11,15 +10,11 @@ import com.unciv.logic.battle.ICombatant
 import com.unciv.logic.battle.MapUnitCombatant
 import com.unciv.logic.battle.TargetHelper
 import com.unciv.logic.city.City
-import com.unciv.logic.civilization.Civilization
 import com.unciv.logic.civilization.NotificationCategory
 import com.unciv.logic.civilization.diplomacy.DiplomaticStatus
-import com.unciv.logic.civilization.managers.ReligionState
 import com.unciv.logic.map.mapunit.MapUnit
 import com.unciv.logic.map.tile.Tile
-import com.unciv.models.UnitActionType
 import com.unciv.models.ruleset.unique.UniqueType
-import com.unciv.ui.screens.worldscreen.unit.actions.UnitActions
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsPillage
 import com.unciv.ui.screens.worldscreen.unit.actions.UnitActionsUpgrade
 
@@ -155,7 +150,7 @@ object UnitAutomation {
 
 
         if (unit.isCivilian()) {
-            automateCivilianUnit(unit)
+            CivilianUnitAutomation.automateCivilianUnit(unit)
             return
         }
 
@@ -174,21 +169,21 @@ object UnitAutomation {
         if ((unit.hasUnique(UniqueType.BuildImprovements) || unit.hasUnique(UniqueType.FoundCity) ||
                 unit.hasUnique(UniqueType.ReligiousUnit) || unit.hasUnique(UniqueType.CreateWaterImprovements))
                 && !unit.civ.isAtWar()){
-            automateCivilianUnit(unit)
+            CivilianUnitAutomation.automateCivilianUnit(unit)
             return
         }
 
         if (unit.baseUnit.isAirUnit() && unit.canIntercept())
-            return SpecificUnitAutomation.automateFighter(unit)
+            return AirUnitAutomation.automateFighter(unit)
 
         if (unit.baseUnit.isAirUnit() && !unit.baseUnit.isNuclearWeapon())
-            return SpecificUnitAutomation.automateBomber(unit)
+            return AirUnitAutomation.automateBomber(unit)
 
         if (unit.baseUnit.isNuclearWeapon())
-            return SpecificUnitAutomation.automateNukes(unit)
+            return AirUnitAutomation.automateNukes(unit)
 
-        if (unit.hasUnique(UniqueType.SelfDestructs))
-            return SpecificUnitAutomation.automateMissile(unit)
+        if (unit.baseUnit.isAirUnit() && unit.hasUnique(UniqueType.SelfDestructs))
+            return AirUnitAutomation.automateMissile(unit)
 
         if (tryGoToRuinAndEncampment(unit) && unit.currentMovement == 0f) return
 
@@ -210,7 +205,7 @@ object UnitAutomation {
         if (tryTakeBackCapturedCity(unit)) return
 
         // Focus all units without a specific target on the enemy city closest to one of our cities
-        if (tryHeadTowardsEnemyCity(unit)) return
+        if (HeadTowardsEnemyCityAutomation.tryHeadTowardsEnemyCity(unit)) return
 
         if (tryGarrisoningRangedLandUnit(unit)) return
 
@@ -235,127 +230,6 @@ object UnitAutomation {
             wander(unit, stayInTerritory = true)
     }
 
-    private fun automateCivilianUnit(unit: MapUnit) {
-        if (tryRunAwayIfNeccessary(unit)) return
-
-        if (unit.currentTile.isCityCenter() && unit.currentTile.getCity()!!.isCapital()
-                && !unit.hasUnique(UniqueType.AddInCapital)
-                && unit.civ.units.getCivUnits().any { unit.hasUnique(UniqueType.AddInCapital) }){
-            // First off get out of the way, then decide if you actually want to do something else
-            val tilesCanMoveTo = unit.movement.getDistanceToTiles()
-                .filter { unit.movement.canMoveTo(it.key) }
-            if (tilesCanMoveTo.isNotEmpty())
-                unit.movement.moveToTile(tilesCanMoveTo.minByOrNull { it.value.totalDistance }!!.key)
-        }
-
-        val tilesWhereWeWillBeCaptured = unit.currentTile.getTilesInDistance(5)
-            .mapNotNull { it.militaryUnit }
-            .filter { it.civ.isAtWarWith(unit.civ) }
-            .flatMap { it.movement.getReachableTilesInCurrentTurn() }
-            .filter { it.militaryUnit?.civ != unit.civ }
-            .toSet()
-
-        if (unit.hasUnique(UniqueType.FoundCity))
-            return SpecificUnitAutomation.automateSettlerActions(unit, tilesWhereWeWillBeCaptured)
-
-        if (unit.cache.hasUniqueToBuildImprovements)
-            return unit.civ.getWorkerAutomation().automateWorkerAction(unit, tilesWhereWeWillBeCaptured)
-
-        if (unit.cache.hasUniqueToCreateWaterImprovements){
-            if (!unit.civ.getWorkerAutomation().automateWorkBoats(unit))
-                tryExplore(unit)
-            return
-        }
-
-        if (unit.hasUnique(UniqueType.MayFoundReligion)
-                && unit.civ.religionManager.religionState < ReligionState.Religion
-                && unit.civ.religionManager.mayFoundReligionAtAll()
-        )
-            return SpecificUnitAutomation.foundReligion(unit)
-
-        if (unit.hasUnique(UniqueType.MayEnhanceReligion)
-                && unit.civ.religionManager.religionState < ReligionState.EnhancedReligion
-                && unit.civ.religionManager.mayEnhanceReligionAtAll(unit)
-        )
-            return SpecificUnitAutomation.enhanceReligion(unit)
-
-        // We try to add any unit in the capital we can, though that might not always be desirable
-        // For now its a simple option to allow AI to win a science victory again
-        if (unit.hasUnique(UniqueType.AddInCapital))
-            return SpecificUnitAutomation.automateAddInCapital(unit)
-
-        //todo this now supports "Great General"-like mod units not combining 'aura' and citadel
-        // abilities, but not additional capabilities if automation finds no use for those two
-        if (unit.cache.hasStrengthBonusInRadiusUnique
-                && SpecificUnitAutomation.automateGreatGeneral(unit))
-            return
-        if (unit.cache.hasCitadelPlacementUnique && SpecificUnitAutomation.automateCitadelPlacer(unit))
-            return
-        if (unit.cache.hasCitadelPlacementUnique || unit.cache.hasStrengthBonusInRadiusUnique)
-            return SpecificUnitAutomation.automateGreatGeneralFallback(unit)
-
-        if (unit.civ.religionManager.maySpreadReligionAtAll(unit))
-            return SpecificUnitAutomation.automateMissionary(unit)
-
-        if (unit.hasUnique(UniqueType.PreventSpreadingReligion) || unit.canDoLimitedAction(Constants.removeHeresy))
-            return SpecificUnitAutomation.automateInquisitor(unit)
-
-        val isLateGame = isLateGame(unit.civ)
-        // Great scientist -> Hurry research if late game
-        if (isLateGame) {
-            val hurriedResearch = UnitActions.invokeUnitAction(unit, UnitActionType.HurryResearch)
-            if (hurriedResearch) return
-        }
-
-        // Great merchant -> Conduct trade mission if late game and if not at war.
-        // TODO: This could be more complex to walk to the city state that is most beneficial to
-        //  also have more influence.
-        if (unit.hasUnique(UniqueType.CanTradeWithCityStateForGoldAndInfluence)
-                // Don't wander around with the great merchant when at war. Barbs might also be a
-                // problem, but hopefully by the time we have a great merchant, they're under control.
-                && !unit.civ.isAtWar()
-                && isLateGame
-        ) {
-            val tradeMissionCanBeConductedEventually =
-                    SpecificUnitAutomation.conductTradeMission(unit)
-            if (tradeMissionCanBeConductedEventually)
-                return
-        }
-
-        // Great engineer -> Try to speed up wonder construction if late game
-        if (isLateGame &&
-                (unit.hasUnique(UniqueType.CanSpeedupConstruction)
-                        || unit.hasUnique(UniqueType.CanSpeedupWonderConstruction))) {
-            val wonderCanBeSpedUpEventually = SpecificUnitAutomation.speedupWonderConstruction(unit)
-            if (wonderCanBeSpedUpEventually)
-                return
-        }
-
-
-        // This has to come after the individual abilities for the great people that can also place
-        // instant improvements (e.g. great scientist).
-        if (unit.hasUnique(UniqueType.ConstructImprovementInstantly)
-        ) {
-            // catch great prophet for civs who can't found/enhance/spread religion
-            // includes great people plus moddable units
-            val improvementCanBePlacedEventually =
-                    SpecificUnitAutomation.automateImprovementPlacer(unit)
-            if (!improvementCanBePlacedEventually)
-                UnitActions.invokeUnitAction(unit, UnitActionType.StartGoldenAge)
-        }
-
-        // TODO: The AI tends to have a lot of great generals. Maybe there should be a cutoff
-        //  (depending on number of cities) and after that they should just be used to start golden
-        //  ages?
-
-        return // The AI doesn't know how to handle unknown civilian units
-    }
-
-    private fun isLateGame(civ: Civilization): Boolean {
-        val researchCompletePercent =
-                (civ.tech.researchedTechnologies.size * 1.0f) / civ.gameInfo.ruleset.technologies.size
-        return researchCompletePercent >= 0.8f
-    }
 
     /** @return true only if the unit has 0 movement left */
     private fun tryAttacking(unit: MapUnit): Boolean {
@@ -389,21 +263,7 @@ object UnitAutomation {
 
         val currentUnitTile = unit.getTile()
 
-        val nearbyRangedEnemyUnits = unit.currentTile.getTilesInDistance(3)
-                .flatMap { tile -> tile.getUnits().filter { unit.civ.isAtWarWith(it.civ) } }
-
-        val tilesInRangeOfAttack = nearbyRangedEnemyUnits
-                .flatMap { it.getTile().getTilesInDistance(it.getRange()) }
-
-        val tilesWithinBombardmentRange = unit.currentTile.getTilesInDistance(3)
-                .filter { it.isCityCenter() && it.getCity()!!.civ.isAtWarWith(unit.civ) }
-                .flatMap { it.getTilesInDistance(it.getCity()!!.range) }
-
-        val tilesWithTerrainDamage = unit.currentTile.getTilesInDistance(3)
-                .filter { unit.getDamageFromTerrain(it) > 0 }
-
-        val dangerousTiles = (tilesInRangeOfAttack + tilesWithinBombardmentRange + tilesWithTerrainDamage).toHashSet()
-
+        val dangerousTiles = getDangerousTiles(unit)
 
         val viableTilesForHealing = unitDistanceToTiles.keys
                 .filter { it !in dangerousTiles && unit.movement.canMoveTo(it) }
@@ -449,6 +309,23 @@ object UnitAutomation {
         return true
     }
 
+    private fun getDangerousTiles(unit: MapUnit): HashSet<Tile> {
+        val nearbyRangedEnemyUnits = unit.currentTile.getTilesInDistance(3)
+            .flatMap { tile -> tile.getUnits().filter { unit.civ.isAtWarWith(it.civ) } }
+
+        val tilesInRangeOfAttack = nearbyRangedEnemyUnits
+            .flatMap { it.getTile().getTilesInDistance(it.getRange()) }
+
+        val tilesWithinBombardmentRange = unit.currentTile.getTilesInDistance(3)
+            .filter { it.isCityCenter() && it.getCity()!!.civ.isAtWarWith(unit.civ) }
+            .flatMap { it.getTilesInDistance(it.getCity()!!.range) }
+
+        val tilesWithTerrainDamage = unit.currentTile.getTilesInDistance(3)
+            .filter { unit.getDamageFromTerrain(it) > 0 }
+
+        return (tilesInRangeOfAttack + tilesWithinBombardmentRange + tilesWithTerrainDamage).toHashSet()
+    }
+
     fun tryPillageImprovement(unit: MapUnit): Boolean {
         if (unit.isCivilian()) return false
         val unitDistanceToTiles = unit.movement.getDistanceToTiles()
@@ -540,114 +417,6 @@ object UnitAutomation {
         return unit.currentMovement == 0f
     }
 
-    fun tryHeadTowardsEnemyCity(unit: MapUnit): Boolean {
-        if (unit.civ.cities.isEmpty()) return false
-
-        // only focus on *attacking* 1 enemy at a time otherwise you'll lose on both fronts
-
-        val enemies = unit.civ.getKnownCivs()
-            .filter { unit.civ.isAtWarWith(it) && it.cities.isNotEmpty() }
-
-        val closestEnemyCity = enemies
-            .mapNotNull { NextTurnAutomation.getClosestCities(unit.civ, it) }
-            .minByOrNull { it.aerialDistance }?.city2
-          ?: return false // no attackable cities found
-
-        // Our main attack target is the closest city, but we're fine with deviating from that a bit
-        var enemyCitiesByPriority = closestEnemyCity.civ.cities
-            .associateWith { it.getCenterTile().aerialDistanceTo(closestEnemyCity.getCenterTile()) }
-            .asSequence().filterNot { it.value > 10 } // anything 10 tiles away from the target is irrelevant
-            .sortedBy { it.value }.map { it.key } // sort the list by closeness to target - least is best!
-
-        if (unit.baseUnit.isRanged()) // ranged units don't harm capturable cities, waste of a turn
-            enemyCitiesByPriority = enemyCitiesByPriority.filterNot { it.health == 1 }
-
-        val closestReachableEnemyCity = enemyCitiesByPriority
-                .firstOrNull { unit.movement.canReach(it.getCenterTile()) }
-
-        if (closestReachableEnemyCity != null) {
-            return headTowardsEnemyCity(
-                unit,
-                closestReachableEnemyCity.getCenterTile(),
-                // This should be cached after the `canReach` call above.
-                unit.movement.getShortestPath(closestReachableEnemyCity.getCenterTile())
-            )
-        }
-        return false
-    }
-
-
-    private fun headTowardsEnemyCity(
-        unit: MapUnit,
-        closestReachableEnemyCity: Tile,
-        shortestPath: List<Tile>
-    ): Boolean {
-        val unitDistanceToTiles = unit.movement.getDistanceToTiles()
-        val unitRange = unit.getRange()
-
-        if (unitRange > 2) { // long-ranged unit, should never be in a bombardable position
-            val tilesInBombardRange = closestReachableEnemyCity.getTilesInDistance(2).toSet()
-            val tileToMoveTo =
-                    unitDistanceToTiles.asSequence()
-                            .filter {
-                                it.key.aerialDistanceTo(closestReachableEnemyCity) <=
-                                        unitRange && it.key !in tilesInBombardRange
-                                        && unit.getDamageFromTerrain(it.key) <= 0 // Don't set up on a mountain
-                            }
-                        .minByOrNull { it.value.totalDistance }?.key
-
-            // move into position far away enough that the bombard doesn't hurt
-            if (tileToMoveTo != null) {
-                unit.movement.headTowards(tileToMoveTo)
-                return true
-            }
-            return false
-        }
-
-        // None of the stuff below is relevant if we're still quite far away from the city, so we
-        // short-circuit here for performance reasons.
-        val minDistanceFromCityToConsiderForLandingArea = 3
-        val maxDistanceFromCityToConsiderForLandingArea = 5
-        if (unit.currentTile.aerialDistanceTo(closestReachableEnemyCity) > maxDistanceFromCityToConsiderForLandingArea
-                // Even in the worst case of only being able to move 1 tile per turn, we would still
-                // not overshoot.
-                && shortestPath.size > minDistanceFromCityToConsiderForLandingArea ) {
-            unit.movement.moveToTile(shortestPath[0])
-            return true
-        }
-
-        val ourUnitsAroundEnemyCity = closestReachableEnemyCity.getTilesInDistance(6)
-            .flatMap { it.getUnits() }
-            .filter { it.isMilitary() && it.civ == unit.civ }
-
-        val city = closestReachableEnemyCity.getCity()!!
-        val cityCombatant = CityCombatant(city)
-
-        val expectedDamagePerTurn = ourUnitsAroundEnemyCity
-            .map { BattleDamage.calculateDamageToDefender(MapUnitCombatant(it), cityCombatant) }
-            .sum() // City heals 20 per turn
-
-        if (expectedDamagePerTurn < city.health && // If we can take immediately, go for it
-            (expectedDamagePerTurn <= 20 || city.health / (expectedDamagePerTurn-20) > 5)){ // otherwise check if we can take within a couple of turns
-
-            // We won't be able to take this even with 5 turns of continuous damage!
-            // don't head straight to the city, try to head to landing grounds -
-            // this is against tha AI's brilliant plan of having everyone embarked and attacking via sea when unnecessary.
-            val tileToHeadTo = closestReachableEnemyCity.getTilesInDistanceRange(3..5)
-                    .filter { it.isLand && unit.getDamageFromTerrain(it) <= 0 } // Don't head for hurty terrain
-                    .sortedBy { it.aerialDistanceTo(unit.currentTile) }
-                    .firstOrNull { (unit.movement.canMoveTo(it) || it == unit.currentTile) && unit.movement.canReach(it) }
-
-            if (tileToHeadTo != null) { // no need to worry, keep going as the movement alg. says
-                unit.movement.headTowards(tileToHeadTo)
-            }
-            return true
-        }
-
-        unit.movement.moveToTile(shortestPath[0]) // go for it!
-
-        return true
-    }
 
     fun tryEnterOwnClosestCity(unit: MapUnit): Boolean {
         val closestCity = unit.civ.cities
@@ -712,7 +481,7 @@ object UnitAutomation {
                 .firstOrNull { unit.movement.canReach(it) }
 
         if (closestReachableCapturedCity != null) {
-            return headTowardsEnemyCity(
+            return HeadTowardsEnemyCityAutomation.headTowardsEnemyCity(
                 unit,
                 closestReachableCapturedCity,
                 // This should be cached after the `canReach` call above.
@@ -720,7 +489,6 @@ object UnitAutomation {
             )
         }
         return false
-
     }
 
     private fun tryGarrisoningRangedLandUnit(unit: MapUnit): Boolean {
@@ -731,14 +499,6 @@ object UnitAutomation {
                     && unit.movement.canMoveTo(centerTile)
         }
 
-        fun isCityThatNeedsDefendingInWartime(city: City): Boolean {
-            if (city.health < city.getMaxHealth()) return true // this city is under attack!
-            for (enemyCivCity in unit.civ.diplomacy.values
-                    .filter { it.diplomaticStatus == DiplomaticStatus.War }
-                    .map { it.otherCiv() }.flatMap { it.cities })
-                if (city.getCenterTile().aerialDistanceTo(enemyCivCity.getCenterTile()) <= 5) return true // this is an edge city that needs defending
-            return false
-        }
 
         val citiesToTry = if (!unit.civ.isAtWar()) {
             if (unit.getTile().isCityCenter()) return true // It's always good to have a unit in the city center, so if you haven't found anyone around to attack, forget it.
@@ -760,6 +520,15 @@ object UnitAutomation {
         return true
     }
 
+    private fun isCityThatNeedsDefendingInWartime(city: City): Boolean {
+        if (city.health < city.getMaxHealth()) return true // this city is under attack!
+        for (enemyCivCity in city.civ.diplomacy.values
+            .filter { it.diplomaticStatus == DiplomaticStatus.War }
+            .map { it.otherCiv() }.flatMap { it.cities })
+            if (city.getCenterTile().aerialDistanceTo(enemyCivCity.getCenterTile()) <= 5) return true // this is an edge city that needs defending
+        return false
+    }
+
     private fun tryStationingMeleeNavalUnit(unit: MapUnit): Boolean {
         fun isMeleeNaval(mapUnit: MapUnit) = mapUnit.baseUnit.isMelee() && mapUnit.type.isWaterUnit()
 
@@ -799,54 +568,8 @@ object UnitAutomation {
         unit.action = null
     }
 
-    /** Returns whether the civilian spends its turn hiding and not moving */
-    private fun tryRunAwayIfNeccessary(unit: MapUnit): Boolean {
-        // This is a little 'Bugblatter Beast of Traal': Run if we can attack an enemy
-        // Cheaper than determining which enemies could attack us next turn
-        val enemyUnitsInWalkingDistance = unit.movement.getDistanceToTiles().keys
-            .filter { containsEnemyMilitaryUnit(unit, it) }
 
-        if (enemyUnitsInWalkingDistance.isNotEmpty() && !unit.baseUnit.isMilitary()
-            && unit.getTile().militaryUnit == null && !unit.getTile().isCityCenter()) {
-            runAway(unit)
-            return true
-        }
-
-        return false
-    }
-
-    private fun runAway(unit: MapUnit) {
-        val reachableTiles = unit.movement.getDistanceToTiles()
-        val enterableCity = reachableTiles.keys
-            .firstOrNull { it.isCityCenter() && unit.movement.canMoveTo(it) }
-        if (enterableCity != null) {
-            unit.movement.moveToTile(enterableCity)
-            return
-        }
-        val defensiveUnit = reachableTiles.keys
-            .firstOrNull {
-                it.militaryUnit != null && it.militaryUnit!!.civ == unit.civ && it.civilianUnit == null
-            }
-        if (defensiveUnit != null) {
-            unit.movement.moveToTile(defensiveUnit)
-            return
-        }
-        val tileFurthestFromEnemy = reachableTiles.keys
-            .filter { unit.movement.canMoveTo(it) && unit.getDamageFromTerrain(it) < unit.health }
-            .maxByOrNull { countDistanceToClosestEnemy(unit, it) }
-            ?: return // can't move anywhere!
-        unit.movement.moveToTile(tileFurthestFromEnemy)
-    }
-
-
-    private fun countDistanceToClosestEnemy(unit: MapUnit, tile: Tile): Int {
-        for (i in 1..3)
-            if (tile.getTilesAtDistance(i).any { containsEnemyMilitaryUnit(unit, it) })
-                return i
-        return 4
-    }
-
-    private fun containsEnemyMilitaryUnit(unit: MapUnit, tile: Tile) =
+    internal fun containsEnemyMilitaryUnit(unit: MapUnit, tile: Tile) =
         tile.militaryUnit != null
         && tile.militaryUnit!!.civ.isAtWarWith(unit.civ)
 
